<?php
/**
 * @copyright Copyright (C) eZ Systems AS. All rights reserved.
 * @license For full copyright and license information view LICENSE file distributed with this source code.
 */
namespace BD\EzPlatformGraphQLBundle\Command;

use BD\EzPlatformGraphQLBundle\GraphQL\TypeDefinition\ContentTypeMapper;
use eZ\Publish\API\Repository\Repository;
use eZ\Publish\API\Repository\Values\ContentType\ContentType;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Serializer\NameConverter\CamelCaseToSnakeCaseNameConverter;
use Symfony\Component\Yaml\Yaml;

class GeneratePlatformDomainTypesCommand extends Command
{
    private $repository;

    /**
     * @var ContentTypeMapper
     */
    private $mapper;

    public function __construct(Repository $repository, ContentTypeMapper $mapper)
    {
        parent::__construct();
        $this->repository = $repository;
        $this->mapper = $mapper;
    }

    protected function configure()
    {
        $this->setName('bd:platform-graphql:generate-domain-schema');
    }


    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $converter = new CamelCaseToSnakeCaseNameConverter(null, false);

        $typesDirectory = "src/AppBundle/Resources/config/graphql";
        $contentTypeService = $this->repository->getContentTypeService();

        $schema = [
            'DomainContentTypesList' => [
                'type' => 'object',
                'config' => [
                    'fields' => []
                ]
            ],
        ];

        $fs = new Filesystem();
        foreach ($contentTypeService->loadContentTypeGroups() as $contentTypeGroup)
        {
            foreach ($contentTypeService->loadContentTypes($contentTypeGroup) as $contentType) {
                $graphQLTypeName = $converter->denormalize($contentType->identifier) . 'Content';
                $typeFilePath = "$typesDirectory/$graphQLTypeName.types.yml";
                $fs->dumpFile(
                    $typeFilePath,
                    Yaml::dump([$graphQLTypeName => $this->mapper->mapContentType($contentType)], 6)
                );
                $output->writeln("Written $typeFilePath");

                $this->addContentTypeToSchema(
                    $schema['DomainContentTypesList']['config']['fields'],
                    $contentType,
                    $graphQLTypeName
                );
            }
        }


        // this should write the list down
        $fs->dumpFile(
            "{$typesDirectory}/platform_domain.types.yml",
            Yaml::dump($schema, 6)
        );

        $output->writeln('Platform domain types generated to src/AppBundle/Resources/config/graphql/platform_domain.types.yml');
    }

    /**
     * @param array $fields
     * @param ContentType $contentType
     * @param $graphQLTypeName
     * @return mixed
     */
    protected function addContentTypeToSchema(array &$fields, ContentType $contentType, $graphQLTypeName)
    {
        $descriptions = $contentType->getDescriptions();
        $fields[$contentType->identifier] = [
            'type' => "[$graphQLTypeName]",
            'description' => isset($descriptions['eng-GB']) ? $descriptions['eng-GB'] : 'No description available',
            'resolve' => sprintf(
                '@=resolver("DomainContentItemsByTypeIdentifier", ["%s"])',
                $contentType->identifier
            ),
        ];
    }

    private function mapFieldTypeIdentifierToGraphQLType($fieldTypeIdentifier)
    {
        return isset($this->typesMap[$fieldTypeIdentifier]) ? $this->typesMap[$fieldTypeIdentifier] : 'GenericFieldValue';
    }
}
